@@ -1,11 +1,13 @@
 from abc import abstractmethod
 import asyncio
-from typing import Dict, Optional
+from typing import Dict, Optional, Callable
 from pydantic import BaseModel, root_validator
 from graphql import parse
+import uuid
+import random
 
 
-class PortInputTrait(BaseModel):
+class PortTrait(BaseModel):
     """
     Class for validating port input
     on the client side
@@ -35,6 +37,40 @@ class PortInputTrait(BaseModel):
 
         return values
 
+    def mock(
+        self,
+        structure_generator: Callable = uuid.uuid4,
+        int_generator: Callable = lambda: random.randint(0, 100),
+        float_generator: Callable = lambda: random.random(),
+        string_generator: Callable = lambda: str("sss"),
+    ):
+        """
+        Mocks some serialized data for this port
+        """
+        from rekuest.api.schema import PortKind
+
+        kind = self.kind
+
+        if kind == PortKind.STRUCTURE:
+            return str(structure_generator())
+
+        if kind == PortKind.LIST:
+            return [self.child.mock()]
+
+        if kind == PortKind.DICT:
+            return {"hello": self.child.mock(), "world": self.child.mock()}
+
+        if kind == PortKind.STRING:
+            return string_generator()
+
+        if kind == PortKind.INT:
+            return int_generator()
+
+        if kind == PortKind.BOOL:
+            return float_generator()
+
+        return None
+
 
 class WidgetInputTrait(BaseModel):
     """
